from openopt.kernel.ooMisc import norm
from numpy import dot, asfarray, atleast_1d,  zeros, ones, float64, where, inf, ndarray, flatnonzero
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.nonOptMisc import isspmatrix, scipyInstalled, scipyAbsentMsg, isPyPy
from lsqr import lsqr as LSQR

try:
    from scipy.sparse import csc_matrix, csr_matrix
except:
    pass

class lsqr(baseSolver):
    __name__ = 'lsqr'
    __license__ = "GPL?"
    __authors__ = 'Michael P. Friedlander (University of British Columbia), Dominique Orban (Ecole Polytechnique de Montreal)'
    __alg__ = 'an iterative (conjugate-gradient-like) method'
    __info__ = """    
    Parameters: atol (default 1e-9), btol (1e-9), conlim ('autoselect', default 1e8 for LLSP and 1e12 for SLE)
    
    For further information, see 

    1. C. C. Paige and M. A. Saunders (1982a).
       LSQR: An algorithm for sparse linear equations and sparse least squares,
       ACM TOMS 8(1), 43-71.
    2. C. C. Paige and M. A. Saunders (1982b).
       Algorithm 583.  LSQR: Sparse linear equations and least squares problems,
       ACM TOMS 8(2), 195-209.
    3. M. A. Saunders (1995).  Solution of sparse rectangular systems using
       LSQR and CRAIG, BIT 35, 588-604."""
       
    __optionalDataThatCanBeHandled__ = ['damp', 'X']
    _canHandleScipySparse = True
    atol = 1e-9
    btol = 1e-9
    conlim = 'autoselect'
    
    def __init__(self): pass

    def __solver__(self, p):
        condX = hasattr(p, 'X') and any(p.X)
        if condX: 
            p.err("sorry, the solver can't handle non-zero X data yet, but you can easily handle it by yourself")
        C, d = p.C, p.d
        m, n = C.shape[0], p.n
        
        if scipyInstalled:
            if isspmatrix(C) or 0.25* C.size > flatnonzero(C).size:
                C = csc_matrix(C)
        elif not isPyPy and 0.25* C.size > flatnonzero(C).size:
            p.pWarn(scipyAbsentMsg)
            
#         if isinstance(C, ndarray) and 0.25* C.size > flatnonzero(C).size:
#            if not scipyInstalled: 
#                p.pWarn(scipyAbsentMsg)
#            else:
#                C = csc_matrix(C)
                
        CT = C.T
        
        def aprod(mode, m, n, x):
            if mode == 1:
                r = dot(C, x).flatten() if not isspmatrix(C) else C._mul_sparse_matrix(csr_matrix(x.reshape(x.size, 1))).A.flatten()
                
                # It doesn't implemented properly yet
#                f = p.norm(r-d)
#                assert damp == 0
#                if damp != 0: 
#                    assert not condX
#                p.iterfcn(x)
#                if p.istop: raise isSolved
                return r
            elif mode == 2:
                return dot(CT, x).flatten() if not isspmatrix(C) else CT._mul_sparse_matrix(csr_matrix(x.reshape(x.size, 1))).A.flatten()
            
        if self.conlim == 'autoselect':
            conlim = 1e12 if m == n else 1e8
            
        damp = self.damp if hasattr(self, 'damp') and self.damp is not None else 0
        show = False
        
        [ x, istop, itn, r1norm, r2norm, anorm, acond, arnorm, xnorm, var ] = \
        LSQR(m, n, aprod, d, damp, 1e-9, 1e-9, conlim, p.maxIter, show, wantvar = False, callback = p.iterfcn)
        # ( m, n, aprod, b, damp, atol, btol, conlim, itnlim, show, wantvar = False )

        #p.istop, p.msg, p.iter = istop, msg.rstrip(), iter
        p.istop = 1000
        p.debugmsg('lsqr iterations elapsed: %d' % itn)
        #p.iter = 1 # itn
        p.xf = x
        #p.ff = p.fk = p.objFunc(x)




"""
Example:
Solving nonsmooth problem
|x1| + 1.2|x2| + 1.44|x3| + ... + 1.2^N |xN| -> min
N=75
x0 = [cos(1), cos(2), ..., cos(N)]
x_opt = all-zeros
f_opt = 0
"""

from numpy import *
from openopt import NSP

#N = 12
#K = 11
N = 1000
K = 10

#N = 10
#K = 7

P = 1.0002
P2 = 1.001
a = 3.5

#N = 7
#K = 6

#objFun = lambda x: sum(1.2 ** arange(len(x)) * abs(x))
#objFun = lambda x: sum(1.2 ** arange(len(x)) * x**2)

f1 = lambda x: sum(abs(x) ** P)
f2 = lambda x: sum(a ** (3+arange(K)) * abs(x[:K])**P2)
f = lambda x: f1(x) + f2(x)

df1 = lambda x: P * sign(x) * abs(x) ** (P-1) 
df2 = lambda x: hstack((a ** (3+arange(K)) * P2 * abs(x[:K]) ** (P2-1)*sign(x[:K]), zeros(N-K)))
df = lambda x: df1(x) + df2(x)

#f, df = f1, df1

#objFun = lambda x: sum(x**2)

x0 = cos(1+asfarray(range(N)))


#OPTIONAL: user-supplied gradient/subgradient
#p.df = lambda x: 1.2 ** arange(len(x)) * 2*x#sign(x)


#p.df = lambda x: 2*x
    
#p.plot = 0
#p.xlim = (inf,  5)
#p.ylim = (0, 5000000)
#p.checkdf()
solvers = ['r2', 'ipopt', 'algencan','ralg']
solvers = ['r2', 'algencan','ralg']
#solvers = ['ralg', 'r2']
solvers = ['r2', 'lincher']
solvers = ['ralg']
#solvers = ['r2']
#solvers = ['scipy_slsqp']
#solvers = ['algencan']
#solvers = ['ipopt']
colors = ['r', 'b', 'k', 'g']
maxIter = 1000
for i, solver in enumerate(solvers):
    p = NSP(f, x0, df=df, xtol = 1e-7, maxIter = maxIter, maxTime=150, ftol=1e-7)
    #p.checkdf()
    r = p.solve(solver, maxVectorNum=35, iprint=1, showLS=0, plot=0, color=colors[i], show=solver==solvers[-1]) # ralg is name of a solver
#p = NSP(f, x0=r.xf, df=df, xtol = 1e-7, maxIter = maxIter, maxTime=150, ftol=1e-7)
#r = p.solve(solver, maxVectorNum=45, iprint=1, showLS=0, plot=0, color=colors[i], show=solver==solvers[-1])
#for i, solver in enumerate(solvers):
#    p2 = NSP(f, r.xf, df=df, xtol = 1e-6, maxIter = 1200, maxTime=150, ftol=1e-6)
#    #p.checkdf()
#    r2 = p2.solve(solver, maxVectorNum=15, iprint=1, showLS=1, plot=0, color=colors[i], show=solver==solvers[-1]) # ralg is name of a solver
#print 'x_opt:\n', r.xf
print 'f_opt:', r.ff  # should print small positive number like 0.00056



from ooMisc import assignScript
from nonOptMisc import isspmatrix
from baseProblem import MatrixProblem
from numpy import asarray, ones, inf, dot, nan, zeros, isnan, any, vstack, array, asfarray
from ooMisc import norm

class LCP(MatrixProblem):
    _optionalData = []
    expectedArgs = ['M', 'q']
    goal = 'solve'
    probType = 'LCP'
    allowedGoals = ['solve']
    showGoal = False

    def __init__(self, *args, **kwargs):
        MatrixProblem.__init__(self, *args, **kwargs)
        self.x0 = zeros(2*len(self.q))
        
    def objFunc(self, x):
        return norm(dot(self.M, x[x.size/2:]) +self.q - x[:x.size/2], inf)




from numpy import log
from openopt import NLP
x0 = [4, 5, 6]
#h = lambda x: log(1+abs(4+x[1]))
#f = lambda x: log(1+abs(x[0]))
f = lambda x: x[0]**4 + x[1]**4 + x[2]**4
df = lambda x: [4*x[0]**3,  4*x[1]**3, 4*x[2]**3]
h = lambda x: [(x[0]-1)**2,  (x[1]-1)**4]
dh = lambda x: [[2*(x[0]-1), 0, 0],  [0, 4*(x[1]-1)**3, 0]]
colors = ['r', 'b', 'g', 'k', 'y']
solvers = ['ralg','scipy_cobyla', 'algencan', 'ipopt', 'scipy_slsqp']
solvers = ['ralg','algencan']
contol = 1e-8
gtol = 1e-8

for i, solver in enumerate(solvers):
    p = NLP(f, x0, df=df, h=h, dh=dh, gtol = gtol, diffInt = 1e-1, contol = contol,  iprint = 1000, maxIter = 1e5, maxTime = 50, maxFunEvals = 1e8, color=colors[i], plot=0, show = i == len(solvers))
    p.checkdh()
    r = p.solve(solver)

#
#x0 = 4
##h = lambda x: log(1+abs(4+x[1]))
##f = lambda x: log(1+abs(x[0]))
#f = lambda x: x**4
#h = lambda x: (x-1)**2
#colors = ['r', 'b', 'g', 'k', 'y']
#solvers = ['ralg','scipy_cobyla', 'algencan', 'ipopt', 'scipy_slsqp']
##solvers = ['algencan']
#contol = 1e-8
#gtol = 1e-8
#for i, solver in enumerate(solvers):
#    p = NLP(f, x0, h=h, gtol = gtol, contol = contol,  iprint = 1000, maxIter = 1e5, maxTime = 50, maxFunEvals = 1e8, color=colors[i], plot=0, show = i == len(solvers))
#    r = p.solve(solver)



from NLOPT_AUX import NLOPT_AUX
from NLOPT_BASE import NLOPT_BASE
import nlopt

class sbplx(NLOPT_BASE):
    __name__ = 'sbplx'
    __alg__ = 'a variant of Nelder-Mead that uses Nelder-Mead on a sequence of subspaces'
    __authors__ = 'Steven G. Johnson'
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __isIterPointAlwaysFeasible__ = lambda self, p: True
    
    funcForIterFcnConnection = 'f'
    
    def __init__(self): pass
    def __solver__(self, p):
        #NLOPT_AUX(p, nlopt.LN_BOBYQA)
        #NLOPT_AUX(p, nlopt.LN_COBYLA)
        NLOPT_AUX(p, nlopt.LN_SBPLX)
        #NLOPT_AUX(p, nlopt.LD_MMA)



from ooMisc import assignScript
from baseProblem import MatrixProblem
from numpy import asfarray, ones, inf, dot, nan, zeros, any, isfinite, sign
from ooMisc import norm
import NSP

class LLAVP(MatrixProblem):
    _optionalData = ['damp', 'X', 'c']
    allowedGoals = ['minimum', 'min']
    def __init__(self, *args, **kwargs):
        if len(args) > 2: self.err('incorrect args number for LLAVP constructor, must be 0..2 + (optionaly) some kwargs')
        if len(args) > 0: kwargs['C'] = args[0]
        if len(args) > 1: kwargs['d'] = args[1]

        MatrixProblem.__init__(self)
        llavp_init(self, kwargs)

    def objFunc(self, x):
        r = norm(dot(self.C, x) - self.d, 1)
        if not self.damp is None:
            r += self.damp * norm(x-self.X, 1)
        #if any(isfinite(self.f)): r += dot(self.f, x)
        return r

    def llavp2nsp(self, solver, **solver_params):
        if hasattr(self,'x0'): p = NSP.NSP(ff, self.x0, df=dff)
        else: p = NSP.NSP(ff, zeros(self.n), df=dff)
        p.args.f = self # DO NOT USE p.args = self IN PROB ASSIGNMENT!
        self.fill(p)
        self.iprint = -1
        # for LLAVP plot is via NLP
        p.show = self.show
        p.plot, self.plot = self.plot, 0
        #p.checkdf()
        #p.solver.__optionalDataThatCanBeHandled__ += ['damp', 'X', 'c']
        r = p.solve(solver, **solver_params)
        self.xf, self.ff, self.rf = r.xf, r.ff, r.rf
        return r

    def _Prepare(self):
        MatrixProblem._Prepare(self)
        if not self.damp is None and not any(isfinite(self.X)):
            self.X = zeros(self.n)




def llavp_init(prob, kwargs):

    prob.probType = 'LLAVP'
    prob.goal = 'minimum'
    prob.allowedGoals = ['minimum', 'min']
    prob.showGoal = False

    kwargs['C'] = asfarray(kwargs['C'])

    prob.n = kwargs['C'].shape[1]
    prob.lb = -inf * ones(prob.n)
    prob.ub =  inf * ones(prob.n)
    if 'damp' not in kwargs.keys(): kwargs['damp'] = None
    if 'X' not in kwargs.keys(): kwargs['X'] = nan*ones(prob.n)

    if prob.x0 is None: prob.x0 = zeros(prob.n)

    return assignScript(prob, kwargs)

ff = lambda x, LLAVprob: LLAVprob.objFunc(x)
def dff(x, LLAVprob):
    r = dot(sign(dot(LLAVprob.C, x) - LLAVprob.d), LLAVprob.C)
    #r = (LLAVprob.C * sign(dot(LLAVprob.C, x) - LLAVprob.d)).sum(0)
    if not LLAVprob.damp is None: r += LLAVprob.damp * (sign(x - LLAVprob.X)).sum(0)
    return r



#!/usr/bin/python
'''
Simplest OpenOpt KSP example;
requires FuncDesigner installed.
For some solvers limitations on time, cputime, "enough" value, basic GUI features are available.
See http://openopt.org/KSP for more details
'''
from openopt import *
from numpy import sin, cos

N = 15

items = [{'name': 'item %d' % i,'weight': 1.5*(cos(i)+1)**2, 'volume': 2*sin(i) + 3, 'n': 2} for i in range(N)]
constraints = lambda values: (
                              values['volume'] < 10, 
                              values['volume'] > 5, 
                              values['nItems'] >= N/5, 
                              values['nItems'] <= 2*N/3
                              )

objective = [
              # name, tol, goal
              'volume', 1.0, 'min', 
              'weight', 0.5, 'max'
              # you could use custom funcs, e.g.
              #lambda v: 2*v['weight'] - 3*v['volume'] + 4 * v['nItems']
              # With interalg you could use nonlinear funcs, including FuncDesigner splines, e.g. 
              #lambda v: 2*v['weight'] - 3*v['volume']**2 + 4 * v['nItems'], 0.01, 'max'
              ]

p = KSP(objective, items, constraints = constraints, name = 'ksp_mop') 
r = p.solve('interalg', plot=1, iprint = 1) 
# see r.solutions, r.solutions.coords, r.solutions.values




from NLOPT_AUX import NLOPT_AUX
from NLOPT_BASE import NLOPT_BASE
import nlopt

class auglag(NLOPT_BASE):
    __name__ = 'auglag'
    __alg__ = "Augmented Lagrangian"
    
    __optionalDataThatCanBeHandled__ = ['lb', 'ub', 'c', 'h','A', 'b', 'Aeq', 'beq']
    
    #TODO: check it!
    #_canHandleScipySparse = True
    
    #funcForIterFcnConnection = 'f'
    
    
    def __init__(self): pass
    def __solver__(self, p):
        NLOPT_AUX(p, nlopt.LD_AUGLAG)



#!/usr/bin/python
'''
Simplest OpenOpt KSP example;
requires FuncDesigner installed.
For some solvers limitations on time, cputime, "enough" value, basic GUI features are available.
See http://openopt.org/KSP for more details
'''
from openopt import *
from numpy import sin, cos

N = 150

items = [
         {
             'name': 'item %d' % i, # pay attention that Python indexation starts from zero
             'cost': 1.5*(cos(i)+1)**2, 
             'volume': 2*sin(i) + 3, 
             'mass': 4*cos(i)+5,
             'n':  1 if i < N/3 else 2 if i < 2*N/3 else 3 # number of elements
         } 
         for i in range(N) # i = 0, ... , N-1
         ]
         
constraints = lambda values: (
                              values['volume'] < 10, 
                              values['mass'] < 100,
                              values['nItems'] <= 10, 
                              values['nItems'] >= 5
                              # we could use lambda-func, e,g.
                              # values['mass'] + 4*values['volume'] < 100
                              )
objective = 'cost'
# we could use lambda-func, e.g. 
# objective = lambda val: 5*value['cost'] - 2*value['volume'] - 5*value['mass'] + 3*val['nItems']
p = KSP(objective, items, goal = 'max', constraints = constraints) 
r = p.solve('glpk', iprint = 0) # requires cvxopt and glpk installed, see http://openopt.org/KSP for other solvers
''' Results for Intel Atom 1.6 GHz:
------------------------- OpenOpt 0.50 -------------------------
solver: glpk   problem: unnamed    type: MILP   goal: max
 iter   objFunVal   log10(maxResidual)   
    0  0.000e+00               0.70 
    1  2.739e+01            -100.00 
istop: 1000 (optimal)
Solver:   Time Elapsed = 0.82   CPU Time Elapsed = 0.82
objFunValue: 27.389749 (feasible, MaxResidual = 0)
'''
print(r.xf) # {'item 131': 2, 'item 18': 1, 'item 62': 2, 'item 87': 1, 'item 43': 1}
# pay attention that Python indexation starts from zero: item 0, item 1 ...
# if fields 'name' are absent in items, you'll have list of numbers instead of Python dict



PythonMax = max
from numpy import ndarray, atleast_1d, isfinite, all, any, string_, zeros_like, inf, \
ones, array_equal, array, ravel
from fdmisc import formDictOfFixedFuncs, linear_render, formDictOfRedirectedFuncs

from nonOptMisc import EmptyClass, isPyPy, oosolver, Hstack, Vstack
#from oologfcn import OpenOptException
from fdConstraint import fdConstraint

def fdPrepare(p):
    p.isFDmodel = True
    p._FD = EmptyClass()
    p._FD.nonBoxConsWithTolShift = []
    p._FD.nonBoxCons = []
    from FuncDesigner import _getAllAttachedConstraints, _getDiffVarsID, ooarray, oopoint, oofun
    from FuncDesigner.distribution import Stochastic
    
    p._FDVarsID = _getDiffVarsID()
    
    probDep = set()
    

    
    if p.probType in ['SLE', 'NLSP', 'SNLE', 'LLSP']:
        equations = p.C if p.probType in ('SLE', 'LLSP') else p.f
        F = equations
        updateDep(probDep, equations)
        ConstraintTags = [(elem if not isinstance(elem, (set, list, tuple, ndarray)) else elem[0]).isConstraint for elem in equations]
        cond_all_oofuns_but_not_cons = not any(ConstraintTags) 
        cond_cons = all(ConstraintTags) 
        if not cond_all_oofuns_but_not_cons and not cond_cons:
            p.err('for FuncDesigner SLE/SNLE constructors args must be either all-equalities or all-oofuns')            
        if p.fTol is not None:
            fTol = min((p.ftol, p.fTol))
            p.warn('''
            both ftol and fTol are passed to the SNLE;
            minimal value of the pair will be used (%0.1e);
            also, you can modify each personal tolerance for equation, e.g. 
            equations = [(sin(x)+cos(y)=-0.5)(tol = 0.001), ...]
            ''' % fTol)
        else:
            fTol = p.ftol
        p.fTol = p.ftol = fTol
        appender = lambda arg: [appender(elem) for elem in arg] if isinstance(arg, (ndarray, list, tuple, set))\
        else ((arg.oofun*(fTol/arg.tol) if arg.tol != fTol and arg.tol != 0 else arg.oofun) if arg.isConstraint else arg)
        EQs = []
        for eq in equations:
            rr = appender(eq)
            if type(rr) == list:
                EQs += rr
            else:
                EQs.append(rr)
        #EQs = [((elem.oofun*(fTol/elem.tol) if elem.tol != 0 else elem.oofun) if elem.isConstraint else elem) for elem in equations]
        if p.probType in ('SLE', 'LLSP'): 
            p.C = EQs
        elif p.probType in ('NLSP', 'SNLE'): 
            p.f = EQs
#                    p.user.F = EQs
        else: 
            p.err('bug in OO kernel')
    else:
        F = [p.f]
        updateDep(probDep, p.f)
        
    probDep2 = probDep.copy()
    updateDep(probDep, p.constraints)
    updateDepWithInvolvedVariables(probDep2, p.constraints)
    
    unInvolvedVariables = probDep.difference(probDep2) # that have only box constraints
    
    
    
    # TODO: implement it
    
#            startPointVars = set(p.x0.keys())
#            D = startPointVars.difference(probDep)
#            if len(D):
#                print('values for variables %s are missing in start point' % D)
#            D2 = probDep.difference(startPointVars)
#            if len(D2):
#                p.x0 = dict([(key, p.x0[key]) for key in D2])

    for fn in ['lb', 'ub', 'A', 'Aeq', 'b', 'beq']:
        if not hasattr(p, fn): continue
        val = getattr(p, fn)
        if val is not None and any(isfinite(val)):
            p.err('while using oovars providing lb, ub, A, Aeq for whole prob is forbidden, use for each oovar instead')
            
    if not isinstance(p.x0, dict):
        p.err('Unexpected start point type: ooPoint or Python dict expected, '+ str(type(p.x0)) + ' obtained')
    
    x0 = p.x0.copy()

    tmp = []
    p._init_ooarrays = set()
    p._init_fixed_ooarrays = set()
    
    for key, val in x0.items():
        if type(key) == ooarray:
            p._init_ooarrays.add(key)
        if isinstance(val, Stochastic):
            p._isStochastic = True
        if not isinstance(key, (list, tuple, ndarray)):
            tmp.append((key, val))
        else: # can be ooarray 
            val = atleast_1d(val)
            if len(key) != val.size:
                p.err('''
                for the sake of possible bugs prevention lenght of oovars array 
                must be equal to lenght of its start point value, 
                assignments like x = oovars(m); startPoint[x] = 0 are forbidden, 
                use startPoint[x] = [0]*m or np.zeros(m) instead''')
            for i in range(val.size):
                tmp.append((key[i], val[i]))
    Tmp = dict(tmp)
    
    if isinstance(p.fixedVars, dict):
        for key, val in p.fixedVars.items():
            if type(key) == ooarray:
                p._init_fixed_ooarrays.add(key)
            if isinstance(key, (list, tuple, ndarray)): # can be only ooarray 
                if type(val) not in (list, tuple, ndarray) or len(key) != len(val):
                    p.err('''
                    for the sake of possible bugs prevention lenght of oovars array 
                    must be equal to lenght of its start point value, 
                    assignments like x = oovars(m); fixedVars[x] = 0 are forbidden, 
                    use fixedVars[x] = [0]*m or np.zeros(m) instead''')
                for i in range(len(val)):
                    Tmp[key[i]] = val[i]
            else:
                Tmp[key] = val
        p.fixedVars = set(p.fixedVars.keys())
    # mb other operations will speedup it?
    if p.probType != 'ODE':
        Keys = set(Tmp.keys()).difference(probDep)
        for key in Keys:
            Tmp.pop(key)
            
    p.probDep = probDep
    x0 = p.x0 = Tmp
    p._aux_fixedVars = dict((v, x0[v]) for v in unInvolvedVariables)
    p._stringVars = set()
    for key, val in x0.items():
        #if key.domain is not None and key.domain is not bool and key.domain is not 'bool':
        if type(val) in (str, string_):
            p._stringVars.add(key)
            key.formAuxDomain()
            x0[key] = key.aux_domain[val]#searchsorted(key.aux_domain, val, 'left')
        elif key.fields == () and key.domain is not None and key.domain is not bool and key.domain is not 'bool' \
        and key.domain is not int and key.domain is not 'int' and val not in key.domain:
            x0[key] = key.domain[0]
    
    x0 = p.x0 = oopoint(x0)
    x0.maxDistributionSize = p.maxDistributionSize

    from fdTranslators import setStartVectorAndTranslators
    setStartVectorAndTranslators(p)
    
    F_ = p.C if p.probType in ('SLE', 'LLSP') else p.f
    
    f_elems = [F_] if isinstance(F_, oofun) else F_.view(ndarray) if isinstance(F_, ooarray) else F 

    # TODO: rework it for MOP
    f_order = inf if p.probType=='MOP' else PythonMax(elem.getOrder(p.freeVarsSet, p.fixedVarsSet, fixedVarsScheduleID = p._FDVarsID) for elem in f_elems)
    #print('f_order:', f_order)
    
    if p.probType in ['LP', 'MILP', 'SOCP'] and f_order > 1:
        p.err('for LP/MILP objective function has to be linear, while this one ("%s") is not' % p.f.name)
    
    if p.fixedVars is None:
       D_kwargs = {'fixedVars':p.fixedVarsSet}
    elif p.freeVars is not None and len(p.freeVars)<len(p.fixedVars):
        D_kwargs = {'Vars':p.freeVarsSet}
    else:
        D_kwargs = {'fixedVars':p.fixedVarsSet}
    D_kwargs['useSparse'] = p.useSparse
    D_kwargs['fixedVarsScheduleID'] = p._FDVarsID
    D_kwargs['exactShape'] = True
    
    p._D_kwargs = D_kwargs
    
    variableTolerancesDict = dict((v, v.tol) for v in p._freeVars)
    p.variableTolerances = p._point2vector(variableTolerancesDict)
    
    if len(p._fixedVars) < len(p._freeVars) and 'isdisjoint' in dir(set()):
        areFixed = lambda dep: dep.issubset(p.fixedVarsSet)
        #isFixed = lambda v: v in p._fixedVars
        Z = dict((v, zeros_like(val) if v not in p.fixedVarsSet else val) for v, val in p._x0.items())
        _isFixedNonStoch = lambda t: t in p.fixedVarsSet and not isinstance(p._x0[t], Stochastic)#could be improved
    else:
        areFixed = lambda dep: dep.isdisjoint(p.freeVarsSet)
        #isFixed = lambda v: v not in p._freeVars
        Z = dict((v, zeros_like(val) if v in p.freeVarsSet else val) for v, val in p._x0.items())
        _isFixedNonStoch = lambda t: t not in p.freeVarsSet and not isinstance(p._x0[t], Stochastic)#could be improved
    p._isFixedNonStoch = _isFixedNonStoch
    p._areFixed = areFixed
    Z = oopoint(Z, maxDistributionSize = p.maxDistributionSize)
    p._Z = Z
   
    #p.isFixed = isFixed
    lb, ub = -inf*ones(p.n), inf*ones(p.n)

    # TODO: get rid of start c, h = None, use [] instead
    A, b, Aeq, beq = [], [], [], []
    
    if type(p.constraints) not in (list, tuple, set):
        p.constraints = [p.constraints]
    oovD = p._oovarsIndDict
    LB = {}
    UB = {}
    
    """                                    gather attached constraints                                    """
    C = list(p.constraints)
    p._initial_constraints = C
    p.constraints = set(p.constraints)
    for v in p._x0.keys():
#                if v.fields != ():
#                    v.aux_domain = Copy(v.domain)
##                    # TODO: mb rework it
##                    ind_numeric = [j for j, elem in enumerate(v.aux_domain[0]) if type(elem) not in (str, np.str_)]
##                    if len(ind_numeric):
##                        ind_first_numeric = ind_numeric[0]
##                        v.aux_domain.sort(key = lambda elem: elem[ind_first_numeric])
#                    v.domain = np.arange(len(v.domain))
        if not array_equal(v.lb, -inf):
            p.constraints.add(v >= v.lb)
        if not array_equal(v.ub, inf):
            p.constraints.add(v <= v.ub)            
            
    if p.useAttachedConstraints: 
        if hasattr(p, 'f'):
            if type(p.f) in [list, tuple, set]:
                C += list(p.f)
            else: # p.f is oofun
                C.append(p.f)
        p.constraints.update(_getAllAttachedConstraints(C))

    FF = p.constraints.copy()
    for _F in F:
        if isinstance(_F, (tuple, list, set)):
            FF.update(_F)
        elif isinstance(_F, ndarray):
            if _F.size > 1:
                FF.update(_F)
            else:
                FF.add(_F.item())
        else:
            FF.add(_F)
    unvectorizableFuncs = set()

    #unvectorizableVariables = set([var for var, val in p._x0.items() if isinstance(val, _Stochastic) or asarray(val).size > 1])
    
    # TODO: use this
    unvectorizableVariables = set([])
    
    # temporary replacement:
    #unvectorizableVariables = set([var for var, val in p._x0.items() if asarray(val).size > 1])
    
    
    cond = False
    #debug
#            unvectorizableVariables = set(p._x0.keys())
#            hasVectorizableFuncs = False
#            cond = True
    #debug end
    if 1 and isPyPy:
        hasVectorizableFuncs = False
        unvectorizableFuncs = FF
    else:
        hasVectorizableFuncs = False
        if len(unvectorizableVariables) != 0:
            for ff in FF:
                _dep = ff._getDep()
                if cond or len(_dep & unvectorizableVariables) != 0:
                    unvectorizableFuncs.add(ff)
                else:
                    hasVectorizableFuncs = True
        else:
            hasVectorizableFuncs = True
    p.unvectorizableFuncs = unvectorizableFuncs
    p.hasVectorizableFuncs = hasVectorizableFuncs
    
    for v in p.freeVarsSet:
        d = v.domain
        if d is bool or d is 'bool':
            p.constraints.update([v>0, v<1])
        elif d is not None and d is not int and d is not 'int':
            # TODO: mb add integer domains?
            v.domain = array(list(d))
            v.domain.sort()
            p.constraints.update([v >= v.domain[0], v <= v.domain[-1]])
            if hasattr(v, 'aux_domain'):
                p.constraints.add(v <= len(v.aux_domain)-1)
            
#            for v in p._stringVars:
#                if isFixed(v):
#                    ind = searchsorted(v.aux_domain, p._x0[v], 'left')
#                    if v.aux_domain

    """                                         handling constraints                                         """
    StartPointVars = set(p._x0.keys())
    p._dictOfStochVars = dict((k, v) for k, v in p._x0.items() if isinstance(v, Stochastic))
    p.dictOfFixedFuncs = {}
    p._dictOfRedirectedFuncs = {}#p._dictOfStochVars
#    p._init_ooarrays = set()
    hasFixedVariables = len(p.fixedVarsSet)
    
    from FuncDesigner import broadcast
    if hasFixedVariables:
        for item in f_elems:
           broadcast(formDictOfFixedFuncs, item, p.useAttachedConstraints, p)
    if p._isStochastic:
        broadcast(formDictOfRedirectedFuncs, f_elems, p.useAttachedConstraints, p)
#    broadcast(getInitOOArrays, f_elems, p.useAttachedConstraints, p)
        
    if oosolver(p.solver).useLinePoints:
        p._firstLinePointDict = {}
        p._secondLinePointDict = {}
        p._currLinePointDict = {}
        
    inplaceLinearRender = oosolver(p.solver).__name__ == 'interalg'
    
    if inplaceLinearRender and hasattr(p, 'f'):
        D_kwargs2 = D_kwargs.copy()
        D_kwargs2['useSparse'] = False
        if type(p.f) in [list, tuple, set]:
            ff = []
            for f in p.f:
                if f.getOrder(p.freeVarsSet, p.fixedVarsSet, fixedVarsScheduleID = p._FDVarsID) < 2:
                    D = f.D(Z, **D_kwargs2)
                    f2 = linear_render(f, D, Z)
                    ff.append(f2)
                else:
                    ff.append(f)
            p.f = ff
        else: # p.f is oofun
            if f_order < 2:
                D = p.f.D(Z, **D_kwargs2)
                p.f = linear_render(p.f, D, Z)
                if p.isObjFunValueASingleNumber:
                    p._linear_objective = True
                    p._linear_objective_factor = p._pointDerivative2array(D).flatten()
                    p._linear_objective_scalar = p.f(Z)
                    
    handleConstraint_args = (StartPointVars, areFixed, oovD, A, b, Aeq, beq, Z, D_kwargs, LB, UB, inplaceLinearRender)
    
    for c in p.constraints:
        if isinstance(c, ooarray):
            for elem in c: 
                fdConstraint(p, elem, *handleConstraint_args) 
        elif c is True:
            continue
        elif c is False:
            p.err('one of elements from constraints is "False", solution is impossible')
        elif not hasattr(c, 'isConstraint'): 
            p.err('The type ' + str(type(c)) + ' is inappropriate for problem constraints')
        else:
            fdConstraint(p, c, *handleConstraint_args)

    '''                                              Handling A, Aeq, lb, ub                                                         '''
    if len(b) != 0:
        p.A, p.b = Vstack(A), Hstack([ravel(elem) for elem in b])
        if hasattr(p.b, 'toarray'): p.b = p.b.toarray()
    if len(beq) != 0:
        p.Aeq, p.beq = Vstack(Aeq), Hstack([ravel(elem) for elem in beq])
        if hasattr(p.beq, 'toarray'): p.beq = p.beq.toarray()
    for vName, vVal in LB.items():
        inds = oovD[vName]
        lb[inds[0]:inds[1]] = vVal
    for vName, vVal in UB.items():
        inds = oovD[vName]
        ub[inds[0]:inds[1]] = vVal
    p.lb, p.ub = lb, ub
    


##########################################################
def updateDep(Dep, elem):
    from FuncDesigner import oofun
    
    return [updateDep(Dep, f) for f in elem] if isinstance(elem, (tuple, list, set))\
    else [updateDep(Dep, f) for f in atleast_1d(elem)] if isinstance(elem, ndarray)\
    else Dep.update(set([elem]) if elem.is_oovar \
    else elem._getDep()) if isinstance(elem, oofun)\
    else None

def updateDepWithInvolvedVariables(Dep, elem): 
    # skips oovars that have only box bounds and are not involved in 
    # neither objective nor other constraints
    
    from FuncDesigner import oofun
    from FuncDesigner.constraints import BoxBoundConstraint
#    if type(elem) == list:
#        print elem
#    else:
#        print elem.expr, isinstance(elem, BoxBoundConstraint)
#    print '-----'
#    
    
    return [updateDepWithInvolvedVariables(Dep, f) for f in elem] if isinstance(elem, (tuple, list, set))\
    else [updateDepWithInvolvedVariables(Dep, f) for f in atleast_1d(elem)] if isinstance(elem, ndarray)\
    else Dep.update(set([elem]) if elem.is_oovar \
    else elem._getDep()) if isinstance(elem, oofun) and not isinstance(elem, BoxBoundConstraint)\
    else None



from scipy.optimize import fmin_cg
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.ooMisc import isSolved

class scipy_cg(baseSolver):
    __name__ = 'scipy_cg'
    __license__ = "BSD"
    #__authors__ =
    __alg__ = "nonlinear conjugate gradient algorithm of Polak and Ribiere See Wright, and Nocedal 'Numerical Optimization', 1999, pg. 120-122"
    __info__ = 'unconstrained NLP solver'
    iterfcnConnected = True

    def __init__(self):pass
    def __solver__(self, p):

        def iterfcn(x):
            p.xk, p.fk = x, p.f(x)
            p.iterfcn()
            if p.istop: raise isSolved

        try:
            xf = fmin_cg(p.f, p.x0, fprime=p.df, gtol=p.gtol, disp = 0, maxiter=p.maxIter, callback=iterfcn)
        except isSolved:
            xf = p.xk

        ff = p.f(xf)
        p.xk = p.xf = xf
        p.fk = p.ff = ff
        p.istop = 1000
        #p.iterfcn()




from MATLAB_BASE import MATLAB_BASE

class lsqlin(MATLAB_BASE):
    __name__ = 'lsqlin'
    __alg__ = ''
    __authors__ = ''
#    __isIterPointAlwaysFeasible__ = True
    
    __optionalDataThatCanBeHandled__ = ['lb', 'ub', 'A', 'Aeq', 'b', 'beq']
#    properTextOutput = True
    _canHandleScipySparse = True
    opts = ('Display', 'TolFun', 'MaxIter')
    
    def __init__(self): pass
    def __solver__(self, p):
        x = MATLAB_BASE._Solve(self, p, p.C, p.d, p.A, p.b, p.Aeq, p.beq, p.lb, p.ub, p.x0)
        p.xk = p.xf = x.flatten()



from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X, IS_MAX_ITER_REACHED, SMALL_DELTA_F
from numpy import nan, diff, copy

class goldenSection(baseSolver):
    __name__ = 'goldenSection'
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __license__ = "BSD"
    __authors__ = 'Dmitrey'
    __alg__ = "golden section"
    __info__ = '1-dimensional minimizer for finite box-bound problems'
    __isIterPointAlwaysFeasible__ = lambda self, p: True
    useOOiterfcn = True
    rightLocalization = 0 # stop criterium, should be between 0.0 and 1.0
    leftLocalization = 0 # stop criterium, should be between 0.0 and 1.0
    rightBorderForLocalization = None
    leftBorderForLocalization = None
#    solutionType = 'best'# 'right', 'left' are also allowed

    def __init__(self):pass
    def __solver__(self, p):
        if p.n != 1: p.err('the solver ' + self.__name__ +' can handle singe-variable problems only')
        if not p.__isFiniteBoxBounded__(): p.err('the solver ' + self.__name__ +' requires finite lower and upper bounds')
        if SMALL_DELTA_X in p.kernelIterFuncs.keys(): p.kernelIterFuncs.pop(SMALL_DELTA_X)
        a,  b,  f = copy(p.lb),  copy(p.ub),  p.f
        if self.rightBorderForLocalization is None: self.rightBorderForLocalization = p.ub
        if self.leftBorderForLocalization is None: self.leftBorderForLocalization = p.lb

        if self.leftLocalization * (b - self.leftBorderForLocalization) > b - a:
            p.istop,  p.msg = 23, 'right localization has been reached (objFunc evaluation is skipped)'
        elif self.rightLocalization * (a - self.rightBorderForLocalization) > b - a :
            p.istop,  p.msg = 24, 'left localization has been reached (objFunc evaluation is skipped)'
        if p.istop:
            p.xk,  p.fk = 0.5*(a+b), nan
            return

        #if hasattr(p, )
        #phi = 1.6180339887498949 # (1+sqrt(5)) / 2
        s1 = 0.3819660112501051# (3-sqrt(double(5)))/2
        s2 = 0.6180339887498949# (sqrt(double(5))-1)/2
        x1 = a+s1*(b-a)
        x2 = a+s2*(b-a)
        y1,  y2 = f(x1),  f(x2)
        for i in range(p.maxIter):
            p.xk,  p.fk = 0.5*(x1+x2), 0.5*(y1+y2)
            if y1 <= y2:
                xmin,  ymin = x1,  y1
                b,  x2 = x2,  x1
                x1 = a + s1*(b - a)
                y2, y1 = y1,  f(x1)
            else:
                xmin,  ymin = x2,  y2
                a,  x1 = x1,  x2
                x2 = a + s2*(b - a)
                y1, y2 = y2,  f(x2)
            #TODO: handle fEnough criterium for min(y1, y2), not for 0.5*(y1+y2)
            if self.useOOiterfcn: p.iterfcn()
            if i > 4 and max(p.iterValues.f[-4:]) < min(p.iterValues.f[-4:])  + p.ftol:
                p.xf,  p.ff = xmin,  ymin
                p.istop,  p.msg = SMALL_DELTA_F, '|| F[k] - F[k-1] || < ftol'
            elif -p.xtol < b - a < p.xtol: # p.iterfcn may be turned off
                p.xf,  p.ff = xmin,  ymin
#                if self.solutionType == 'best':
#                    p.xf,  p.ff = xmin,  ymin
#                elif self.solutionType == 'left':
#                    p.xf,  p.ff = x1,  y1
#                elif self.solutionType == 'right':
#                    p.xf,  p.ff = x2,  y2
#                else:
#                    p.err('incorrect solutionType, should be "best", "right" or "left"')
                p.istop,  p.msg = SMALL_DELTA_X, '|| X[k] - X[k-1] || < xtol'
            elif self.leftLocalization * (b - self.leftBorderForLocalization) > b - a:
                p.istop,  p.msg = 25, 'right localization has been reached'
            elif self.rightLocalization * (a - self.rightBorderForLocalization) > b - a :
                p.istop,  p.msg = 26, 'left localization has been reached'
            if y2 > y1:
                p.special.leftXOptBorder = a
                p.special.rightXOptBorder = x2
            else:
                p.special.leftXOptBorder = x1
                p.special.rightXOptBorder = b
            if p.istop:
                return
        p.istop, p.msg = IS_MAX_ITER_REACHED, 'Max Iter has been reached'




'''
Translated from Octave code at: http://www.ecs.shimane-u.ac.jp/~kyoshida/lpeng.htm
and placed under MIT licence by Enzo Michelangeli with permission explicitly
granted by the original author, Prof. Kazunobu Yoshida  

-----------------------------------------------------------------------------
Copyright (c) 2010, Kazunobu Yoshida, Shimane University, and Enzo Michelangeli, 
IT Vision Limited

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
-----------------------------------------------------------------------------

Usage:
 
 optx,zmin,is_bounded,sol,basis = lp(c,A,b)
 
  This program finds a solution of the standard linear programming problem:
    minimize    z = c'x
    subject to  Ax = b, x >= 0
  using the two phase method, where the simplex method is used at each stage.
  Returns the tuple:
    optx: an optimal solution.
    zmin: the optimal value. 
    is_bounded: True if the solution is bounded; False if unbounded.
    sol: True if the problem is solvable; False if unsolvable.
    basis: indices of the basis of the solution.
    
'''
from numpy import *
#from numpy.linalg import norm
#from numpy import dot, asfarray, atleast_1d,  zeros, ones, int, float64, where, inf, ndarray
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.nonOptMisc import isspmatrix, scipyInstalled, scipyAbsentMsg
try:
    from openopt.kernel.nonOptMisc import isPyPy
except:
    pass
from openopt.kernel.ooMisc import xBounds2Matrix
from openopt.kernel.nonOptMisc import Hstack, Vstack, SparseMatrixConstructor, Eye, Diag, DenseMatrixConstructor
#try:
#    from scipy.sparse import csc_matrix, csr_matrix
#except:
#    pass

class pclp(baseSolver):
    __name__ = 'pclp'
    __license__ = "MIT"
    __authors__ = ''
    __alg__ = 'a simplex method implementation'
    __info__ = """    """
       
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub']
    _canHandleScipySparse = True

    
    def __init__(self): pass

    def __solver__(self, p):
        xBounds2Matrix(p)
        n = p.n
        
        
        Ind_unbounded = logical_and(isinf(p.lb), isinf(p.ub))
        ind_unbounded = where(Ind_unbounded)[0]
        n_unbounded = ind_unbounded.size
        
        # Cast linear inequality constraints into linear equality constraints using slack variables
        nLinInEq, nLinEq = p.b.size, p.beq.size
        
        #p.lb, p.ub = empty(n+nLinInEq+n_unbounded), empty(n+nLinInEq+n_unbounded)
        #p.lb.fill(-inf)
        #p.ub.fill(inf)
        
#        for fn in ['_A', '_Aeq']:
#            if hasattr(p, fn): delattr(p, fn)

        # TODO: handle p.useSparse parameter
        
#        if n_unbounded != 0:
#            R = SparseMatrixConstructor((n_unbounded, n))
#            R[range(n_unbounded), ind_unbounded] = 1.0
#            R2 = Diag([1]*nLinInEq+[-1]*n_unbounded)
#            _A = Hstack((Vstack((p.A, R)), R2))
#        else:
#            _A = Hstack((p.A, Eye(nLinInEq)))

        _A = Hstack((p.A, Eye(nLinInEq), -Vstack([p.A[:, i] for i in ind_unbounded]).T if isPyPy else -p.A[:, ind_unbounded]))
        

#        if isspmatrix(_A): 
#            _A = _A.A
        
        # add lin eq cons
        if nLinEq != 0:
            Constructor = SparseMatrixConstructor if scipyInstalled and nLinInEq > 100 else DenseMatrixConstructor
            _A = Vstack((_A, Hstack((p.Aeq, Constructor((nLinEq, nLinInEq)), \
                                                         -Vstack([p.Aeq[:, i] for i in ind_unbounded]).T if isPyPy else -p.Aeq[:, ind_unbounded]))))
            
        if isspmatrix(_A): 
            if _A.size > 0.3 * prod(_A.shape):
                _A = _A.A
            else:
                _A = _A.tolil()
            #_A = _A.A
        
        #p.A, p.b = zeros((0, p.n)), array([])
        #_f = hstack((p.f, zeros(nLinInEq+n_unbounded)))
        
        #_f = Hstack((p.f, zeros(nLinInEq), -p.f[ind_unbounded]))
        #_b = hstack((p.b, [0]*n_unbounded, p.beq))
        
        _f = hstack((p.f, zeros(nLinInEq), -p.f[Ind_unbounded]))
        
        _b = hstack((p.b, p.beq))
        
        
        if p.useSparse is False and isspmatrix(_A): _A = _A.A
        p.debugmsg('handling as sparse: ' + str(isspmatrix(_A)))
        optx,zmin,is_bounded,sol,basis = lp_engine(_f, _A, _b)
        p.xf = optx[:n] -optx[-n:] if len(optx) != 0 else nan
        p.istop = 1000 if p.xf is not nan else -1000
        
#        from openopt  import LP
#        p2 = LP(_f, Aeq=_A.copy(), beq=_b, lb=[0]*_f.size)
#        r=p2.solve('lpSolve', maxIter = 1e4)
#        x_opt_2= r.xf[:n] - r.xf[-n:]
        
        #assert len(optx)>0
        #print linalg.norm(p.xf - x_opt_2)
        #raise 0
        

def lp_engine(c, A, b):
#    c = asarray(c)
#    A = asarray(A)
#    b = asarray(b)

    m,n = A.shape
    ind = b < 0
    if any(ind):
        b = abs(b)
        #A[ind, :] = -A[ind, :] doesn't work for sparse
        if type(A) == ndarray and not isPyPy:
            A[ind] = -A[ind]
        else:
            for i in where(ind)[0]:
                A[i,:] = -A[i,:]
            
    d = -A.sum(axis=0)
    if not isscalar(d) and type(d) != ndarray:
        d = d.A.flatten()
    if not isinstance(d, ndarray): d = d.A.flatten() # d may be dense matrix
    w0 = sum(b)
    # H = [A b;c' 0;d -w0];
    #H = bmat('A b; c 0; d -w0') 
    ''''''
    H = Vstack([     #  The initial _simplex table of phase one
         Hstack([A, atleast_2d(b).T]), # first m rows
         hstack([c, 0.]),   # last-but-one
         hstack([d, -asfarray(w0)])]) # last
    #print sum(abs(Hstack([A, array([b]).T])))

    if isspmatrix(H): H = H.tolil()
    ''''''
    indx = arange(n)
    basis = arange(n, n+m)
    #H, basis, is_bounded = _simplex(H, basis, indx, 1)
    is_bounded = _simplex(H, basis, indx, 1)
    if H[m+1,n] < -1e-10:   # last row, last column
        sol = False
        #print('unsolvable')
        optx = []
        zmin = []
        is_bounded = False
    else:
        sol = True
        j = -1
        
        #NEW
        tmp = H[m+1,:]
        if type(tmp) != ndarray: tmp = tmp.A.flatten()
        ind = tmp > 1e-10
        if any(ind):
            j = where(logical_not(ind))[0]
            H = H[:, j]
            indx = indx[j]
        #Old
#        for i in range(n):
#            j = j+1
#            if H[m+1,j] > 1e-10:
#                H[:,j] = []
#                indx[j] = []
#                j = j-1
        #H(m+2,:) = [] % delete last row
        H = H[0:m+1,:]
        if size(indx) > 0:
        # Phase two
            #H, basis, is_bounded = _simplex(H,basis,indx,2);
            is_bounded = _simplex(H,basis,indx,2)
            if is_bounded:
                optx = zeros(n+m)
                n1,n2 = H.shape
                for i in range(m):
                    optx[basis[i]] = H[i,n2-1]
                # optx(n+1:n+m,1) = []; % delete last m elements
                optx = optx[0:n] 
                zmin = -H[n1-1,n2-1]    #  last row, last column
            else:
                optx = []
                zmin = -Inf
        else:
            optx = zeros(n+m)
            zmin = 0
    return (optx, zmin, is_bounded, sol, basis)  

def _simplex(H,basis,indx,s):
    '''
      [H1,basis,is_bounded] = _simplex(H,basis,indx,s)
      H: simplex table (MODIFIED).
      basis: the indices of basis (MODIFIED).
      indx: the indices of x.
      s: 1 for phase one; 2 for phase two.
      H1: new simplex table.
      is_bounded: True if the solution is bounded; False if unbounded.
    '''
    if s == 1:
        s0 = 2
    elif s == 2:
        s0 = 1
    n1, n2 = H.shape
    sol = False
    while not sol:
        #print 'new Iter'
        # [fm,jp] = min(H(n1,1:n2-1));
        q = H[n1-1, 0:n2-1] # last row, all columns but last
        if type(q) != ndarray: q = q.toarray().flatten()
        jp = argmin(q)
        fm = q[jp]
        if fm >= 0:
            is_bounded = True    # bounded solution
            sol = True
        else:
            # [hm,ip] = max(H(1:n1-s0,jp));
            q = H[0:n1-s0,jp]
            if type(q) != ndarray: q = q.toarray().flatten()
            ip = argmax(q)
            hm = q[ip]
            if hm <= 0:
                is_bounded = False # unbounded solution
                sol = True
            else:
                h1 = empty(n1-s0)
                h1.fill(inf)
                
                # NEW
                tmp = H[:n1-s0,jp]
                if isspmatrix(tmp): tmp = tmp.A.flatten()
                ind = tmp>0
                
                #tmp2 = H[ind,n2-1]/H[ind,jp]
                tmp2 = hstack([H[i,n2-1]/H[i, jp] for i in where(ind)[0]]) if 1 or isPyPy else H[ind,n2-1]/H[ind,jp]
                
                #assert hstack([H[i,n2-1]/H[i, jp] for i in where(ind)[0]]).shape == (H[ind,n2-1]/H[ind,jp]).shape
                if isspmatrix(tmp2): tmp2 = tmp2.A
                
                if isPyPy:
                    for i, val in enumerate(where(ind)[0]):
                        h1[val] = tmp2[i]
                else:
                    h1[atleast_1d(ind)] = tmp2
                
                #OLD
#                for i in range(n1-s0):
#                    if H[i,jp] > 0:
#                        h1[i] = H[i,n2-1]/H[i,jp]
                        
                ip = argmin(h1)
                minh1 = h1[ip]
                basis[ip] = indx[jp]
                if not _pivot(H,ip,jp):
                    raise ValueError("the first parameter is a Singular matrix") 
    return is_bounded

def _pivot(H,ip,jp):
    # H is MODIFIED
    n, m = H.shape
    piv = H[ip,jp]
    if piv == 0:
        #print('singular')
        return False
    else:
        #NEW
        H[ip,:] /= piv
        tmp2 = H[:,jp]*H[ip,:] if isspmatrix(H) else H[:,jp].reshape(-1, 1)*H[ip,:]
        if isspmatrix(tmp2): tmp2 = tmp2.tolil()
        tmp2[ip, :] = 0
        H -= tmp2
        #OLD
#        for i in range(n):
#            if i != ip:
#                H[i,:] -= H[i,jp]*H[ip,:]
    return True


######### Unit test section #########
#
#from numpy.testing import *
#
#def test_lp():
#    probs = [
#        {
#            'A': array([
#                [2.,  5., 3., -1.,  0.,  0.],
#                [3., 2.5, 8.,  0., -1.,  0.],
#                [8.,10.,  4.,  0.,  0., -1.]]),
#            'b': array([185., 155., 600.]),
#            'c': array([4., 8., 3., 0., 0., 0.]),
#            'result': [
#                    array([ 66.25, 0., 17.5, 0., 183.75, 0.]),
#                    317.5,
#                    True,
#                    True,
#                    array([2, 0, 4])            
#                ]
#        }, # add other test cases here...
#    ]
#
#    for prob in probs:
#        optx, zmin, bounded, solvable, basis = lp_engine(prob['c'],prob['A'],prob['b'])
#        expected_res = prob['result']
#        assert_almost_equal(optx, expected_res[0])
#        assert_almost_equal(zmin, expected_res[1])
#        assert_equal(bounded, expected_res[2])
#        assert_equal(solvable, expected_res[3])
#        assert_equal(basis, expected_res[4])
#
#if __name__ == "__main__":
#    run_module_suite()



from numpy import asarray,  ones, all, isfinite, copy, nan, concatenate, array, hstack, vstack, atleast_1d
from openopt.kernel.ooMisc import WholeRepr2LinConst, xBounds2Matrix
#from openopt.kernel.nonOptMisc import Vstack
from cvxopt_misc import *
import cvxopt.solvers as cvxopt_solvers
from cvxopt.base import matrix
from openopt.kernel.setDefaultIterFuncs import SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON,  IS_MAX_ITER_REACHED, IS_MAX_TIME_REACHED, FAILED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON, UNDEFINED
from openopt.kernel.nonOptMisc import Vstack, isspmatrix

def CVXOPT_SOCP_Solver(p, solverName):
    if solverName == 'native_CVXOPT_SOCP_Solver': solverName = None
    cvxopt_solvers.options['maxiters'] = p.maxIter
    cvxopt_solvers.options['feastol'] = p.contol    
    cvxopt_solvers.options['abstol'] = p.ftol
    if p.iprint <= 0:
        cvxopt_solvers.options['show_progress'] = False
        cvxopt_solvers.options['LPX_K_MSGLEV'] = 0
        cvxopt_solvers.options['MSK_IPAR_LOG'] = 0
    xBounds2Matrix(p)
    #FIXME: if problem is search for MAXIMUM, not MINIMUM!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    f = copy(p.f).reshape(-1,1)

    # CVXOPT has some problems with x0 so currently I decided to avoid using the one
    
    Gq, hq = [], []
    C, d, q, s = p.C, p.d, p.q, p.s
    for i in range(len(q)):
        Gq.append(Matrix(Vstack((-atleast_1d(q[i]),-atleast_1d(C[i]) if not isspmatrix(C[i]) else C[i]))))
        hq.append(matrix(hstack((atleast_1d(s[i]), atleast_1d(d[i]))), tc='d'))
    sol = cvxopt_solvers.socp(Matrix(p.f), Gl=Matrix(p.A), hl = Matrix(p.b), Gq=Gq, hq=hq, A=Matrix(p.Aeq), b=Matrix(p.beq), solver=solverName)
    p.msg = sol['status']
    if p.msg == 'optimal' :  p.istop = SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON
    else: p.istop = -100
    if sol['x'] is not None:
        p.xf = asarray(sol['x']).flatten()
        p.ff = sum(p.dotmult(p.f, p.xf))
        
    else:
        p.ff = nan
        p.xf = nan*ones(p.n)



from numpy import asfarray, argmax, inf, log10, max
from numpy.linalg import norm
from openopt.kernel.baseSolver import baseSolver
from openopt import NSP
from string import rjust

class nsmm(baseSolver):
    __name__ = 'nsmm'
    __license__ = "BSD"
    __authors__ = 'Dmitrey Kroshko'
    __alg__ = "based on Naum Z. Shor r-alg"
    iterfcnConnected = True
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    __info__ = """
    Solves mini-max problem
    via minimizing max (F[i]) using NSP solver (default UkrOpt.ralg).

    Can handle user-supplied gradient/subradient (p.df field)
    If the one is not available -
    splitting equations to separate functions is recommended
    (to speedup calculations):
    f = [func1, func2, ...] or f = ([func1, func2, ...)
    """

    def __init__(self):pass
    def __solver__(self, p):
        f = lambda x: max(p.f(x))
        def df(x):
            F = p.f(x)
            ind = argmax(F)
            return p.df(x, ind)

        def iterfcn(*args,  **kwargs):
            p2.primalIterFcn(*args,  **kwargs)

            p.xk = p2.xk.copy()
            p.fk = p2.fk
            p.rk = p2.rk

            p.istop = p2.istop

            if p.istop and p2.rk <= p2.contol:
                p.msg = p2.msg
            p.iterfcn()

        p2 = NSP(f, p.x0, df=df, xtol = p.xtol, ftol = p.ftol, gtol = p.gtol,\
        A=p.A,  b=p.b,  Aeq=p.Aeq,  beq=p.beq,  lb=p.lb,  ub=p.ub, \
        maxFunEvals = p.maxFunEvals, fEnough = p.fEnough, maxIter=p.maxIter, iprint = -1, \
        maxtime = p.maxTime, maxCPUTime = p.maxCPUTime,  noise = p.noise)

        if p.userProvided.c:
            p2.c,  p2.dc = p.c,  p.dc
        if p.userProvided.h:
            p2.h,  p2.dh = p.h,  p.dh


        p2.primalIterFcn,  p2.iterfcn = p2.iterfcn, iterfcn

        r2 = p2.solve('ralg')
        xf = r2.xf
        p.xk = p.xf = xf
        p.fk = p.ff = max(p.f(xf))
        #p.istop is defined in iterfcn



__docformat__ = "restructuredtext en"
from numpy import zeros, ones, copy, isfinite, where, asarray, inf, int64, \
array, dot, ndarray, prod, flatnonzero, max, abs, sqrt, sum, atleast_1d, asscalar
from nonOptMisc import scipyAbsentMsg, scipyInstalled, isspmatrix, Hstack, Vstack, coo_matrix, isPyPy

Copy = lambda arg: asscalar(arg) if type(arg)==ndarray and arg.size == 1 else arg.copy() if hasattr(arg, 'copy') else copy(arg)

try:
    from numpy import linalg
    norm = linalg.norm
except ImportError:
    def norm(x, k=2, *args, **kw):
        if len(args) or len(kw):
            raise ImportError('openopt overload for PyPy numpy linalg.norm cannot handle additional args or kwargs')
        if k == 2:
            return sqrt(sum(asarray(x)**2))
        elif k == inf:
            return max(abs(x))
        elif k == 1:
            return sum(abs(x))
        else:
            raise ImportError('unimplemented')

def Len(arg):
    
    # for PyPy:
    if type(arg) == ndarray:
        if arg.size > 1:
            return arg.size  
        elif arg.size == 1 and atleast_1d(arg)[0] is not None:
            return 1 
        elif arg.size == 0:
            return 0
       
    if type(arg) in [int, float]:
        return 1
    elif arg is None or (isinstance(arg, (list, tuple)) and len(arg) == 0) \
    or (isinstance(arg, ndarray) and arg.size==1 and arg == array(None, dtype=object)):
        return 0
    else:
        return len(arg)

def xBounds2Matrix(p):
    """
    transforms lb - ub bounds into (A, x) <= b, (Aeq, x) = beq conditions
    this func is developed for those solvers that can handle lb, ub only via c(x)<=0, h(x)=0
    """

    #TODO: is reshape/flatten required in newest numpy versions?
    
    # for PyPy
    IndLB, IndUB, IndEQ = \
    isfinite(p.lb) & ~(p.lb == p.ub), \
    isfinite(p.ub) & ~(p.lb == p.ub), \
    p.lb == p.ub

    indLB, indUB, indEQ = \
    where(IndLB)[0], \
    where(IndUB)[0], \
    where(IndEQ)[0]

    nLB, nUB, nEQ = Len(indLB), Len(indUB), Len(indEQ)

    if  nLB>0 or nUB>0:
        if p.useSparse is True or (isspmatrix(p.A) or (scipyInstalled and nLB+nUB>=p.A.shape[0]) and p.useSparse is not False):
            R1 = coo_matrix((-ones(nLB), (range(nLB), indLB)), shape=(nLB, p.n)) if nLB != 0 else zeros((0, p.n))
            R2 = coo_matrix((ones(nUB), (range(nUB), indUB)), shape=(nUB, p.n)) if nUB != 0 else zeros((0, p.n))
        else:
            R1 = zeros((nLB, p.n))
            if isPyPy:
                for i in range(nLB):
                    R1[i, indLB[i]] = -1
            else:
                R1[range(nLB), indLB] = -1
            R2 = zeros((nUB, p.n))
            if isPyPy:
                for i in range(nUB):
                    R2[i, indUB[i]] = -1
            else:
                R2[range(nUB), indUB] = 1
        
        p.A = Vstack((p.A, R1, R2))
        if hasattr(p, '_A'): delattr(p, '_A')
        if isspmatrix(p.A): 
            if asarray(p.A.shape, int64).prod()>10000:
                p.A = p.A.tocsc()
                p._A = p.A
            else:
                p.A = p.A.A
        
        p.b = Hstack((p.b, -p.lb[IndLB], p.ub[IndUB]))

    if nEQ>0:
        
        if p.useSparse is True or (isspmatrix(p.Aeq) or (scipyInstalled and nEQ>=p.Aeq.shape[0]) and p.useSparse is not False):
            R = coo_matrix(([1]*nEQ, (range(nEQ), indEQ)), shape=(nEQ, p.n))
        else:
            R = zeros((nEQ, p.n))
        #raise 0
        p.Aeq = Vstack((p.Aeq, R))
        
        if hasattr(p, '_Aeq'): delattr(p, '_Aeq')
        if isspmatrix(p.Aeq): 
            if prod(p.Aeq.shape)>10000:
                p.Aeq = p.Aeq.tocsc()
                p._Aeq = p.Aeq
            else:
                p.Aeq = p.Aeq.A
            
        p.beq = Hstack((p.beq, p.lb[IndEQ]))
        
    p.lb = -inf*ones(p.n)
    p.ub = inf*ones(p.n)

    # TODO: prevent code clone with baseProblem.py
    nA, nAeq = prod(p.A.shape), prod(p.Aeq.shape) 
    SizeThreshold = 2 ** 15
    if scipyInstalled and p.useSparse is not False:
        from scipy.sparse import csc_matrix
        if nA > SizeThreshold and not isspmatrix(p.A) and flatnonzero(p.A).size < 0.25*nA:
            p._A = csc_matrix(p.A)
        if nAeq > SizeThreshold and not isspmatrix(p.Aeq) and flatnonzero(p.Aeq).size < 0.25*nAeq:
            p._Aeq = csc_matrix(p.Aeq)
    if (nA > SizeThreshold or nAeq > SizeThreshold) and not scipyInstalled and p.useSparse is not False:
        p.pWarn(scipyAbsentMsg)


def LinConst2WholeRepr(p):
    """
    transforms  (A, x) <= b, (Aeq, x) = beq into Awhole, bwhole, dwhole constraints (see help(LP))
    this func is developed for those solvers that can handle linear (in)equality constraints only via Awhole
    """
    if p.A is None and p.Aeq is None:
        return
    
    # new 
    p.Awhole = Vstack([elem for elem in [p.Awhole, p.A, p.Aeq] if elem is not None])
    
    #old
#    Awhole = Copy(p.Awhole) # maybe it's already present and not equal to None
#    p.Awhole = zeros([Len(p.b) + Len(p.beq) + Len(p.bwhole), p.n])
#    if Awhole.size>0: p.Awhole[:Len(p.bwhole)] = Awhole
#    p.Awhole[Len(p.bwhole):Len(p.bwhole)+Len(p.b)] = p.A
#    if p.Aeq.size: p.Awhole[Len(p.bwhole)+Len(p.b):] = p.Aeq
    
    p.A, p.Aeq = None, None

    bwhole = Copy(p.bwhole)
    p.bwhole = zeros(Len(p.b) + Len(p.beq) + Len(p.bwhole))
    p.bwhole[:Len(bwhole)] = bwhole

    p.bwhole[Len(bwhole):Len(bwhole)+Len(p.b)] = p.b

    p.bwhole[Len(bwhole)+Len(p.b):] = p.beq


    dwhole = Copy(p.dwhole)
    p.dwhole = zeros(Len(p.bwhole))
    if dwhole.size: p.dwhole[:Len(bwhole)] = dwhole
    p.dwhole[Len(bwhole):Len(bwhole)+Len(p.b)] = -1
    p.dwhole[Len(bwhole)+Len(p.b):] = 0

    p.b = None
    p.beq = None

def WholeRepr2LinConst(p):
    """
    transforms  Awhole, bwhole, dwhole  into (A, x) <= b, (Aeq, x) = beq constraints (see help(LP))
    this func is developed for those solvers that can handle linear (in)equality constraints only via Awhole
    """
    if p.dwhole is None:
        return
    #TODO: is flatten required in newest numpy versions?
    ind_less = where(p.dwhole == -1)[0]
    ind_greater = where(p.dwhole == 1)[0]
    ind_equal = where(p.dwhole == 0)[0]

    if len(ind_equal) != 0:
        Aeq, beq = Copy(p.Aeq) , Copy(p.beq)
        p.Aeq = zeros([Len(p.beq)+len(ind_equal), p.n])
        if Aeq.size: p.Aeq[:Len(p.beq)] = Aeq
        if len(ind_equal): p.Aeq[Len(p.beq):] = p.Awhole[ind_equal]
        p.beq = zeros([Len(p.beq)+len(ind_equal)])
        if beq.size: p.beq[:Len(beq)] = beq
        if len(ind_equal): p.beq[Len(beq):] = p.bwhole[ind_equal]

    if len(ind_less) + len(ind_greater)>0:
        A, b = Copy(p.A) , Copy(p.b)
        p.A = zeros([Len(p.b)+len(ind_less)+len(ind_greater), p.n])
        if A.size: p.A[:Len(p.b)] = A
        p.A[Len(p.b):Len(p.b)+len(ind_less)] = p.Awhole[ind_less]
        p.A[Len(p.b)+len(ind_less):] = -p.Awhole[ind_greater]
        p.b = zeros(Len(p.b)+len(ind_less)+len(ind_greater))
        if b.size: p.b[:Len(b)] = b
        if len(ind_less): p.b[Len(b):Len(b)+len(ind_less)] = p.bwhole[ind_less]
        if len(ind_greater): p.b[Len(b)+len(ind_less):] = -p.bwhole[ind_greater]

    p.Awhole = None
    p.bwhole = None
    p.dwhole = None

def assignScript(p, dictOfParams):
    for key, val in dictOfParams.items():
        if key == 'manage': 
            #p._useGUIManager = val
            continue
        setattr(p, key, val)
    
def setNonLinFuncsNumber(p,  userFunctionType):
    # userFunctionType  should be 'f', 'c' or 'h'
    args = getattr(p.args, userFunctionType)
    fv = getattr(p.user, userFunctionType)
    
    if p.isFDmodel:
        from FuncDesigner import oopoint
        X = oopoint(p._x0, maxDistributionSize = p.maxDistributionSize)
        kwargs = {'Vars': p.freeVars, 'fixedVarsScheduleID':p._FDVarsID, 'fixedVars': p.fixedVars}
    else:
        X = p.x0
        kwargs = {}

    if isinstance(fv, ndarray):
        fv = fv.tolist()
        
    if len(fv) == 1: 
        p.functype[userFunctionType] = 'single func'
    
    if fv is None or (type(fv) in [list, tuple] and (len(fv)==0 or fv[0] is None)):
        setattr(p, 'n'+userFunctionType, 0)
    elif type(fv) in [list, tuple] and len(fv)>1:
        # TODO: handle problems w/o x0, like GLP
        number = 0
        arr = []
        for func in fv:
            if isinstance(func, (list, tuple, set, ndarray)):
                for elem in func:
                    number += asarray(getVal(elem, X, p, args, kwargs)).size
            else:
                number += asarray(getVal(func, X, p, args, kwargs)).size
            arr.append(number)
            
        if len(arr) < number: 
            p.functype[userFunctionType] = 'block'
        elif len(arr) > 1: 
            p.functype[userFunctionType] = 'some funcs R^nvars -> R'
        else: 
            assert p.functype[userFunctionType] == 'single func'
            
        setattr(p, 'n' + userFunctionType, number)
        
        if p.functype[userFunctionType] == 'block':
            setattr(p, 'arr_of_indexes_' + userFunctionType, array(arr)-1)
    else:
        FV = fv[0] if type(fv) in [list, tuple] else fv
        tmp = getVal(FV, X, p, args, kwargs)
        setattr(p, 'n'+userFunctionType, \
                sum([asarray(elem).size for elem in tmp]) if isinstance(tmp, (list, tuple)) else asarray(tmp).size)

def economyMult(M, V):
    #return dot(M, V)
    assert V.ndim <= 1 or V.shape[1] == 1
    
    if True or all(V) or isPyPy: # all v coords are non-zeros
        return dot(M, V)
    else:
        ind = where(V != 0)[0]
        #ind = V != 0
        r = dot(M[:,ind], V[ind])
        return r

def Find(M):
    if isinstance(M, ndarray): # numpy array or matrix
        rows, cols = where(M)
        vals = M[rows,cols]
    else:
        from scipy import sparse as sp
        assert sp.isspmatrix(M)
        rows, cols, vals = sp.find(M)
    return rows.tolist(), cols.tolist(), vals.tolist()

def getVal(func, X, p, args, kw):
    r = func(*(X,) + args, **kw)
    if p._isStochastic:
        from FuncDesigner import _Stochastic
        if isinstance(r, _Stochastic) or (isinstance(r, ndarray) and isinstance(r.flat[0], _Stochastic)):
            p.err('''
            Optimization problem objective and constraints cannot be of type Stochastic, 
            you can handle only functions on them like mean(X), std(X), var(X), P(X<Y) etc''')
    return r

class isSolved(BaseException):
    def __init__(self): pass
class killThread(BaseException):
    def __init__(self): pass


